@@ -18,41 +18,49 @@
 (* Foundation, Inc.,                                                   *)
 (* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           *)
 (***********************************************************************)
-
-let rec sources_filter param target = function
+ 
+let rec change_unwanted_sources param target = function
   | hd :: tl ->
-      let file = Params.to_change_extension param hd in
-	if file <> target && List.mem file (Params.to_files param) then 
-	  file :: sources_filter param target tl
+      let file =
+	match (Params.to_sorting_strategy param, hd) with
+	  | _, Files.Implementation _ -> hd
+	  | Params.Sort_ml_mli, _ -> hd
+	  | Params.Sort_ml, Files.Interface name ->
+	      Files.Implementation name in
+	if List.mem target (Params.to_files param) && file <> target
+	then
+	  file :: change_unwanted_sources param target tl
 	else
-	  sources_filter param target tl
+	  change_unwanted_sources param target tl
   | [] -> []
 
-let rec clean_dependencies param = function
+let rec change_unwanted_files param = function
   | (target, sources) :: tl ->
-      if (Files.is_cmo target ||
-	  (Files.is_cmi target && Params.to_sort_mli param)) && 
-	List.mem (Params.to_change_extension param target)
-	  (Params.to_files param)
-      then
-	let new_target = Params.to_change_extension param target in
-          (new_target, sources_filter param new_target sources)
-	  :: clean_dependencies param tl
-      else
-	clean_dependencies param tl
+      let keep_target = 
+	match (Params.to_sorting_strategy param, target,
+	       List.mem target (Params.to_files param)) with
+	  | (_, _, false) -> false
+	  | (Params.Sort_ml, Files.Interface _, true) -> false
+	  | (Params.Sort_ml, Files.Implementation _, true) -> true
+      	  | (Params.Sort_ml_mli, _, true) -> true in
+	if keep_target then
+	  (target, change_unwanted_sources param target sources) ::
+	  change_unwanted_files param tl
+	else
+	  change_unwanted_files param tl
   | [] -> []
 
-let rec add_independent_dependencies deps = function 
+let rec add_independent_targets deps = function 
   | hd :: tl -> 
       if List.mem_assoc hd deps then
-	add_independent_dependencies deps tl
+	add_independent_targets deps tl
       else
-	add_independent_dependencies ((hd, []) :: deps) tl
+	add_independent_targets ((hd, []) :: deps) tl
   | [] -> deps
 
+(*   let _ = Dep_debug.print_deps raw_deps in *)
 let get_dependencies param =
   let raw_deps = Dep_parse.parse_ocamldep (Params.to_stream param) in
-(*   let _ = Dep_debug.print_deps raw_deps in *)
-  let cleaned_deps = clean_dependencies param raw_deps in
-(*   let _ = Dep_debug.print_deps cleaned_deps in *)
-    add_independent_dependencies cleaned_deps (Params.to_files param)
+  let complete_deps =
+    add_independent_targets raw_deps (Params.to_files param) in
+    change_unwanted_files param complete_deps
